# -*- coding: utf-8 -*-

#importation de la librarie
import Orange

#version du package
print(Orange.__version__)

#modifier le dossier de travail
import os
os.chdir("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/orange_python")

#charger le fichier de données
D1 = Orange.data.Table("breast_train.xlsx")

#type de l'objet
print(type(D1))

#propriétés
print(dir(D1))

#afficher la liste des variables
print(D1.domain)

#dimension 1 - nb. d'obs.
print(len(D1))

#dimension 2 - nb. de colonnes
print(len(D1[0]))

#afficher les 6 premières lignes
print(D1[:6])

#re-définition du rôle des variables pour le problème à traiter
breast_domain = Orange.data.Domain(D1.domain[:9],D1.domain[9])
print(breast_domain)

#nouveau dataset avec les rôles redéfinis
DTrain = Orange.data.Table(breast_domain,D1)

#vérification
print(DTrain[:6])

#vérif. variable cible
print(DTrain.domain.class_var)

#liste des modalités de la cible
print(DTrain.domain.class_var.values)

#comptage des modalités de la cible
import numpy
numpy.unique(DTrain.Y,return_counts=True)

#liste des variables explicatives
print(DTrain.domain.attributes)

#instancier une régression logistique
learner = Orange.classification.LogisticRegressionLearner()
print(learner)

#apprentissage sur data train
classifier = learner(DTrain)
print(classifier)

#membres de l'objet
dir(classifier)

#constante
print(classifier.intercept)

#coefficients
print(classifier.coefficients)

#coefficients avec les noms des variables
cn = {'variable': [d.name for d in DTrain.domain.attributes], 'coefficient':classifier.coefficients[0]}

#affichage
import pandas
print(pandas.DataFrame.from_dict(cn))

#chargement de l'échantillon test
D2 = Orange.data.Table("breast_test.xlsx")

#re-définition du rôle des variables
DTest = Orange.data.Table(breast_domain,D2)

#comptage des modalités de la cible
numpy.unique(DTest.Y,return_counts=True)

#prédiction
prediction = numpy.array([classifier(DTest,0)])
print(prediction)

#comptage des prédictions 0 et 1
print(numpy.unique(prediction[0],return_counts=True))

#matrice de confusion
mc = pandas.crosstab(DTest.Y,prediction[0])
print(mc)

#taux de succès
acc = numpy.mean(DTest.Y == prediction[0])
print(acc)

#Orange - confrontation classes observées et prédites
res = Orange.evaluation.testing.Results(actual=DTest.Y,predicted=prediction,domain=breast_domain)

#indicateurs de performances

#taux de reconnaissance
print(Orange.evaluation.CA(res))

#print rappel
print(Orange.evaluation.Recall(res))

#print précision
Orange.evaluation.Precision(res)

#F1-Score
print(Orange.evaluation.F1(res))

#algorithme CN2
cn2_learner = Orange.classification.CN2UnorderedLearner()

#paramétrage - nombre d'observations couvertes par chaque règle
cn2_learner.rule_finder.general_validator.min_covered_examples = 20

#apprentissage
cn2_classifier = cn2_learner(DTrain)
print(dir(cn2_classifier))

#affichage des règles
for rule in cn2_classifier.rule_list:
    print(rule)
    
#prédiction en test
cn2_pred = cn2_classifier(DTest)

#taux de reconnaissance - concordance obsv. vs. préd.
print(numpy.mean(DTest.Y == cn2_pred))